

package com.lutao.entity;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.lutao.annotate.Validation;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.time.LocalDateTime;

/**
 * 工艺参数异常闭环管理
 *
 * @author guoyc
 * @date 2023-07-20 10:56:48
 */
@Data
@TableName("facility_param_manager")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "工艺参数异常闭环管理")
public class FacilityParamManager extends Model<FacilityParamManager> {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    @TableId
    @ApiModelProperty(value="主键")
    private Integer id;

    /**
     * 原始id
     */
    @ApiModelProperty(value="原始id")
    @Validation(message = "facility_param_manager表{id}不能为空")
    private String originId;

    /**
     * 设施名称
     */
    @ApiModelProperty(value="设施名称")
    @Validation(message = "facility_param_manager表{facility_name}不能为空")
    private String facilityName;

    /**
     * 设备id
     */
    @ApiModelProperty(value="设备id")
    //@Validation(message = "facility_param_manager表{equip_id}不能为空")
    private String equipId;

    /**
     * 设备编号
     */
    @ApiModelProperty(value="设备编号")
    @Validation(message = "facility_param_manager表{equip_code}不能为空")
    private String equipCode;

    /**
     * 设备名字
     */
    @ApiModelProperty(value="设备名字")
    private String equipName;

    /**
     * 001关键设施设备工艺参数告警 002关键设施设备工况状态报警 003关键设备设备关断状态报警  004关键设施设备报警
     */
    @ApiModelProperty(value="001关键设施设备工艺参数告警 002关键设施设备工况状态报警 003关键设备设备关断状态报警  004关键设施设备报警")
    @Validation(message = "facility_param_manager表{type}不能为空")
    private String type;

    /**
     * 异常状态00已告警、01已报警（0100已停止、0101已关断、0110高高报、0111低低报）
     */
    @ApiModelProperty(value="异常状态00已告警、01已报警（0100已停止、0101已关断、0110高高报、0111低低报）")
    @Validation(message = "facility_param_manager表{state}不能为空")
    private String state;

    /**
     * 报警时间
     */
    @ApiModelProperty(value="报警时间")
    @Validation(message = "facility_param_manager表{alarm_timestamp}不能为空")
    private LocalDateTime alarmTimestamp;

    /**
     * 创建时间
     */
    @ApiModelProperty(value="创建时间")
    @Validation(message = "facility_param_manager表{create_date}不能为空")
    private LocalDateTime createDate;

    /**
     * 创建人姓名
     */
    @ApiModelProperty(value="创建人姓名")
    @Validation(message = "facility_param_manager表{create_by}不能为空")
    private String createBy;

    /**
     * 修改时间
     */
    @ApiModelProperty(value="修改时间")
    @Validation(message = "facility_param_manager表{update_date}不能为空")
    private LocalDateTime updateDate;

    /**
     * 修改人
     */
    @ApiModelProperty(value="修改人")
    @Validation(message = "facility_param_manager表{update_by}不能为空")
    private String updateBy;

    /**
     * 系统平台id
     */
    @ApiModelProperty(value="系统平台id")
    private Integer pfId;

    /**
     * 应用id
     */
    @ApiModelProperty(value="应用id")
    private String appId;

    /**
     * 平台id，平台origin_id
     */
    @ApiModelProperty(value="平台id，平台origin_id")
    private String platformId;

    /**
     * 平台编码
     */
    @ApiModelProperty(value="平台编码")
    @Validation(message = "facility_param_manager表{platform_code}不能为空")
    private String platformCode;

    /**
     * 平台名称
     */
    @ApiModelProperty(value="平台名称")
    private String platformName;

    @ApiModelProperty(value = "数据收集时间")
    @TableField(fill = FieldFill.INSERT)
    private LocalDateTime createTime;

    @ApiModelProperty(value = "数据更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private LocalDateTime updateTime;
}
