package com.lutao.cloud.admin.api.feign;

import com.lutao.cloud.admin.api.dto.UserDTO;
import com.lutao.cloud.admin.api.entity.SysUser;
import com.lutao.cloud.admin.api.dto.UserInfo;
import com.lutao.cloud.admin.api.vo.UserVO;
import com.lutao.cloud.common.core.constant.SecurityConstants;
import com.lutao.cloud.common.core.constant.ServiceNameConstants;
import com.lutao.cloud.common.core.util.MessageInfo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;
import java.util.List;

/**
 * @date 2018/6/22
 */
@FeignClient(contextId = "remoteUserService", value = ServiceNameConstants.UPMS_SERVICE)
public interface RemoteUserService{

    /**
     * 通过用户名查询用户、角色信息
     *
     * @param username 用户名
     * @param from     调用标志
     * @return R
     */
    @GetMapping("/user/info/{username}")
    MessageInfo<UserInfo> info(@PathVariable("username") String username, @RequestHeader(SecurityConstants.FROM) String from);

    /**
     * 通过用户名查询用户信息
     *
     * @param username 用户名
     * @param from     调用标志
     * @return R
     */
    @GetMapping("/user/details/{username}")
    MessageInfo<SysUser> detail(@PathVariable("username") String username, @RequestHeader(SecurityConstants.FROM) String from);



    /**
     * 通过社交账号或手机号查询用户、角色信息
     *
     * @param inStr appid@code
     * @param from  调用标志
     * @return
     */
    @GetMapping("/social/info/{inStr}")
    MessageInfo<UserInfo> social(@PathVariable("inStr") String inStr, @RequestHeader(SecurityConstants.FROM) String from);

    /**
     * 查询上级部门的用户信息
     *
     * @param username 用户名
     * @return R
     */
    @GetMapping("/user/ancestor/{username}")
    MessageInfo<List<SysUser>> ancestorUsers(@PathVariable("username") String username);

    /**
     * 通过用户id查询用户信息
     *
     * @param userId
     * @return R
     */
    @GetMapping("/user/detail/{userId}")
    MessageInfo<UserVO> selectById(@PathVariable("userId") Integer userId, @RequestHeader(SecurityConstants.FROM) String from);

    /**
     * token查询用户权限信息
     * @param username
     * @return
     */
    @GetMapping("/user/userPermission/{username}")
    MessageInfo<UserVO> userPermission(@PathVariable("username") String username, @RequestHeader(SecurityConstants.FROM) String from);

    /**
     * 添加用户信息
     *
     * @param userDto 用户信息
     * @param from  调用标志
     * @return R
     */
    @PostMapping("/user/info")
    MessageInfo<Boolean> saveUser(@RequestBody UserDTO userDto, @RequestHeader(SecurityConstants.FROM) String from);

    /**
     * 更新用户信息
     *
     * @param userDto 用户信息
     * @param from  调用标志
     * @return R
     */
    @PutMapping("/user/info")
    MessageInfo<Boolean> updateUser(@RequestBody UserDTO userDto, @RequestHeader(SecurityConstants.FROM) String from);
}
