package com.lutao.cloud.admin.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.lutao.cloud.admin.api.dto.ForgetPwdDTO;
import com.lutao.cloud.admin.api.dto.RegisterDTO;
import com.lutao.cloud.admin.api.dto.UserDTO;
import com.lutao.cloud.admin.api.dto.UserInfo;
import com.lutao.cloud.admin.api.entity.SysUser;
import com.lutao.cloud.admin.api.vo.UserByIdVO;
import com.lutao.cloud.admin.api.vo.UserExcelVO;
import com.lutao.cloud.admin.api.vo.UserPageVo;
import com.lutao.cloud.admin.api.vo.UserVO;
import com.lutao.cloud.common.core.util.MessageInfo;
import org.springframework.validation.BindingResult;

import java.util.List;

/**
 * @date 2017/10/31
 */
public interface SysUserService extends IService<SysUser>{

    /**
     * 根据用户名查用户权限
     *
     */

    UserVO findUserPermission(String username);

    /**
     * 查询用户信息
     *
     * @param sysUser 用户
     * @return userInfo
     */
    UserInfo findUserInfo(SysUser sysUser);

    /**
     * 分页查询用户信息（含有角色信息）
     *
     * @param page    分页对象
     * @param userDTO 参数列表
     * @return
     */
    IPage getUsersWithRolePage(Page page, UserDTO userDTO);

    /**
     * 删除用户
     *
     * @param sysUser 用户
     * @return boolean
     */
    Boolean deleteUserById(SysUser sysUser);

    /**
     * 更新当前用户基本信息
     *
     * @param userDto 用户信息
     * @return Boolean
     */
    MessageInfo<Boolean> updateUserInfo(UserDTO userDto);


    /**
     * 更新指定用户信息
     *
     * @param userDto 用户信息
     * @return
     */
    Boolean updateUser(UserDTO userDto);

    /**
     * 通过ID查询用户信息
     *
     * @param id 用户ID
     * @return 用户信息
     */
    UserVO selectUserVoById(Integer id);

    /**
     * 查询上级部门的用户信息
     *
     * @param username 用户名
     * @return R
     */
    List<SysUser> listAncestorUsers(String username);

    /**
     * 保存用户信息
     *
     * @param userDto DTO 对象
     * @return success/fail
     */
    MessageInfo<Boolean> saveUser(UserDTO userDto);


    /**
     * 注册
     *
     * @param registerDTO DTO 对象
     * @return success/fail
     */
    MessageInfo<Integer> register(RegisterDTO registerDTO);

    /**
     * 查询全部的用户
     *
     * @param userDTO 查询条件
     * @return list
     */
    List<UserExcelVO> listUser(UserDTO userDTO);

    /**
     * excel 导入用户
     *
     * @param excelVOList   excel 列表数据
     * @param bindingResult 错误数据
     * @return ok fail
     */
    MessageInfo importUser(List<UserExcelVO> excelVOList, BindingResult bindingResult);

    /**
     * 忘记密码
     * @param forgetPwdDTO
     * @return
     */
    MessageInfo forgetPwd(ForgetPwdDTO forgetPwdDTO);

    /**
     * 通过用户id查询用户信息和部门信息
     * @return
     */
    UserByIdVO selectPerson();

    /**
     * 所有用户信息不要密码
     * @param page
     * @return
     */
    MessageInfo<IPage<UserPageVo>> getSelectAll(Page page);
}
